@@ -84,6 +84,7 @@ BowtieHit() :
     _left(0),
     _antisense_splice(false),
     _antisense_aln(false),
+    _mismatches(0),
     _edit_dist(0),
     _splice_mms(0),
     _alignment_score(0),
@@ -95,6 +96,7 @@ BowtieHit(uint32_t ref_id,
 	  int left, 
 	  int read_len, 
 	  bool antisense,
+	  unsigned char mismatches,
 	  unsigned char edit_dist,
 	  bool end) :
   _ref_id(ref_id),
@@ -104,6 +106,7 @@ BowtieHit(uint32_t ref_id,
     _cigar(vector<CigarOp>(1,CigarOp(MATCH,read_len))),
     _antisense_splice(false),
     _antisense_aln(antisense),
+    _mismatches(mismatches),
     _edit_dist(edit_dist),
     _splice_mms(0),
     _alignment_score(0),
@@ -119,6 +122,7 @@ BowtieHit(uint32_t ref_id,
 	  const vector<CigarOp>& cigar,
 	  bool antisense_aln,
 	  bool antisense_splice,
+	  unsigned char mismatches,
 	  unsigned char edit_dist,
 	  unsigned char splice_mms,
 	  bool end) : 
@@ -129,6 +133,7 @@ BowtieHit(uint32_t ref_id,
     _cigar(cigar),
     _antisense_splice(antisense_splice),
     _antisense_aln(antisense_aln),
+    _mismatches(mismatches),
     _edit_dist(edit_dist),
     _splice_mms(splice_mms),
     _alignment_score(0),
@@ -184,6 +189,8 @@ BowtieHit(uint32_t ref_id,
       return _left < rhs._left;
     if (_antisense_aln != rhs._antisense_aln)
       return _antisense_aln < rhs._antisense_aln;
+    if (_mismatches != rhs._mismatches)
+      return _mismatches < rhs._mismatches;
     if (_edit_dist != rhs._edit_dist)
       return _edit_dist < rhs._edit_dist;
     if (_cigar != rhs._cigar)
@@ -421,6 +428,7 @@ BowtieHit(uint32_t ref_id,
       result._antisense_aln = _antisense_aln;
     
     result._antisense_splice = _antisense_splice;
+    result._mismatches = _mismatches;
     result._edit_dist = _edit_dist;
     result._splice_mms = _splice_mms;
     result._end = _end;
@@ -432,9 +440,12 @@ BowtieHit(uint32_t ref_id,
     
     return result;
   }
-  
+
+  unsigned char mismatches() const	{ return _mismatches;		}
+  void mismatches(unsigned char mm) { _mismatches = mm; }
   unsigned char edit_dist() const	{ return _edit_dist;		}
   void edit_dist(unsigned char ed) { _edit_dist = ed; }
+  unsigned char  gap_length() const { return _edit_dist - _mismatches; }
   unsigned char splice_mms() const	{ return _splice_mms;		}
 
   int alignment_score() const           { return _alignment_score;      }
@@ -512,7 +523,8 @@ private:
   
   bool _antisense_splice;    // Whether the junction spanned is on the reverse strand
   bool _antisense_aln;       // Whether the alignment is to the reverse strand
-  
+
+  unsigned char _mismatches;
   unsigned char _edit_dist;  // Total mismatches (note this is not including insertions or deletions as mismatches, ie, not equivalent to NM field of a SAM record)
   unsigned char _splice_mms; // Mismatches within min_anchor_len of a splice junction
   string _hitfile_rec; // Points to the buffer for the record from which this hit came
@@ -564,171 +576,6 @@ inline bool REFID_Equal(uint32_t ref_id1, uint32_t ref_id2)
   return ref_id1 == ref_id2;
 }
 
-#if 0
-class RefSequenceTable
-{
-public:
-	
-	typedef seqan::String<seqan::Dna5, seqan::Packed<seqan::Alloc<> > > Sequence;
-	
-	struct SequenceInfo
-	{
-		SequenceInfo(uint32_t _order, 
-					 char* _name, 
-					 Sequence* _seq, uint32_t _len) :
-            observation_order(_order),
-            name(_name),
-            seq(_seq),
-            len(_len) {}
-        
-		uint32_t observation_order;
-		char* name;
-		Sequence* seq;
-        uint32_t len;
-	};
-	
-	typedef map<string, uint64_t> IDTable;
-	typedef map<uint32_t, SequenceInfo> InvertedIDTable;
-	typedef InvertedIDTable::iterator iterator;
-	typedef InvertedIDTable::const_iterator const_iterator;
-	
-	RefSequenceTable(bool keep_names, bool keep_seqs = false) : 
-	_next_id(1), 
-	_keep_names(keep_names) {}
-    
-    RefSequenceTable(const string& sam_header_filename, 
-                     bool keep_names, 
-                     bool keep_seqs = false) : 
-        _next_id(1), 
-        _keep_names(keep_names) 
-    {
-        if (sam_header_filename != "")
-        {
-            samfile_t* fh = samopen(sam_header_filename.c_str(), "r", 0);
-            if (fh == 0) {
-                fprintf(stderr, "Failed to open SAM header file %s\n", sam_header_filename.c_str());
-                exit(1);
-            }
-            
-            for (size_t i = 0; i < (size_t)fh->header->n_targets; ++i)
-            {
-                const char* name = fh->header->target_name[i];
-                uint32_t len  = fh->header->target_len[i];
-                get_id(name, NULL, len);
-                //fprintf(stderr, "SQ: %s - %d\n", name, len);
-            }
-        }
-    }
-	
-	// This function should NEVER return zero
-	uint32_t get_id(const string& name,
-			Sequence* seq = NULL,
-			uint32_t len = 0)
-	{
-		uint32_t _id = hash_string(name.c_str());
-		pair<InvertedIDTable::iterator, bool> ret = 
-		_by_id.insert(make_pair(_id, SequenceInfo(_next_id, NULL, NULL, 0)));
-		if (ret.second == true)
-		{			
-			char* _name = NULL;
-			if (_keep_names)
-				_name = strdup(name.c_str());
-			ret.first->second.name  = _name;
-			ret.first->second.seq	= seq;
-            ret.first->second.len   = len;
-			++_next_id;
-		}
-		assert (_id);
-		return _id;
-	}
-	
-	const char* get_name(uint32_t ID) const
-	{
-		InvertedIDTable::const_iterator itr = _by_id.find(ID);
-		if (itr != _by_id.end())
-			return itr->second.name;
-		else
-			return NULL;
-	}
-    
-    uint32_t get_len(uint32_t ID) const
-	{
-		InvertedIDTable::const_iterator itr = _by_id.find(ID);
-		if (itr != _by_id.end())
-			return itr->second.len;
-		else
-			return 0;
-	}
-	
-	Sequence* get_seq(uint32_t ID) const
-	{
-		InvertedIDTable::const_iterator itr = _by_id.find(ID);
-		if (itr != _by_id.end())
-			return itr->second.seq;
-		else
-			return NULL;
-	}
-	
-	const SequenceInfo* get_info(uint32_t ID) const
-	{
-		
-		InvertedIDTable::const_iterator itr = _by_id.find(ID);
-		if (itr != _by_id.end())
-		{
-			return &(itr->second);
-		}
-		else
-			return NULL;
-	}
-	
-	int observation_order(uint32_t ID) const
-	{
-		InvertedIDTable::const_iterator itr = _by_id.find(ID);
-		if (itr != _by_id.end())
-		{
-			return itr->second.observation_order;
-		}
-		else
-			return -1;
-	}
-	
-	iterator begin() { return _by_id.begin(); }
-	iterator end() { return _by_id.end(); }
-	
-	const_iterator begin() const { return _by_id.begin(); }
-	const_iterator end() const { return _by_id.end(); }
-	
-	size_t size() const { return _by_id.size(); }
-	
-	void clear()
-	{
-		//_by_name.clear();
-		_by_id.clear();
-	}
-
-	// daehwan
-	// This is FNV-1, see http://en.wikipedia.org/wiki/Fowler_Noll_Vo_hash
-	static inline uint32_t hash_string(const char* __s)
-	{
-		uint32_t hash = 0x811c9dc5;
-		for ( ; *__s; ++__s)
-		{
-			hash *= 16777619;
-			hash ^= *__s;
-		}
-		return hash;
-	}
-	
-private:
-	
-	//IDTable _by_name;
-	uint32_t _next_id;
-	bool _keep_names;
-	InvertedIDTable _by_id;
-};
-
-#else
-
 class RefSequenceTable
 {
  public:
@@ -780,8 +627,17 @@ class RefSequenceTable
                 // fprintf(stderr, "SQ: %s - %u\n", name, len);
             }
 	  // order_recs_lexicographically();
+	  samclose(fh);
         }
     }
+
+  ~RefSequenceTable()
+    {
+      for (IDTable::iterator i = _by_name.begin(); i != _by_name.end(); ++i)
+	{
+	  free(i->second.name);
+	}
+    }
   
   // This function should NEVER return zero
   uint32_t get_id(const string& name,
@@ -930,7 +786,6 @@ private:
   IDTable _by_name;
   vector<string> _refid_to_name;
 };
-#endif
 
 bool hit_insert_id_lt(const BowtieHit& h1, const BowtieHit& h2);
 
@@ -1002,6 +857,7 @@ class HitFactory {
 			 const vector<CigarOp>& cigar,
 			 bool antisense_aln,
 			 bool antisense_splice,
+			 unsigned char mismatches,
 			 unsigned char edit_dist,
 			 unsigned char splice_mms,
 			 bool end);
@@ -1011,6 +867,7 @@ class HitFactory {
 			 uint32_t left,
 			 uint32_t read_len,
 			 bool antisense_aln,
+			 unsigned char mismatches,
 			 unsigned char edit_dist,
 			 bool end);
   
@@ -1058,14 +915,6 @@ class BowtieHitFactory : public LineHitFactory {
 	BowtieHitFactory(ReadTable& insert_table, 
 			RefSequenceTable& reference_table) : 
 		LineHitFactory(insert_table, reference_table) {}
-
-	bool get_hit_from_buf(const char* bwt_buf, 
-			      BowtieHit& bh,
-			      bool strip_slash,
-			      char* name_out = NULL,
-			      char* name_tags = NULL,
-			      char* seq = NULL,
-			      char* qual = NULL);
 };
 
 class SplicedBowtieHitFactory : public LineHitFactory {
@@ -1076,13 +925,6 @@ class SplicedBowtieHitFactory : public LineHitFactory {
 	LineHitFactory(insert_table, reference_table),
 	_anchor_length(anchor_length){}
 	
-	bool get_hit_from_buf(const char* bwt_buf, 
-			      BowtieHit& bh,
-			      bool strip_slash,
-			      char* name_out = NULL,
-			      char* name_tags = NULL,
-			      char* seq = NULL,
-			      char* qual = NULL);
   private:
 	int _anchor_length;
 	int _seg_offset;
@@ -1145,33 +987,23 @@ class BAMHitFactory : public HitFactory {
     void closeStream(HitStream& hs);
 
     bool next_record(HitStream& hs, const char*& buf, size_t& buf_size);
-	
-	/*void mark_curr_pos()
-	{ 
-		_curr_pos = bgzf_tell(((samfile_t*)_hit_file)->x.bam);
-	}
+    string hitfile_rec(HitStream& hs, const char* hit_buf);
+    
+    bool get_hit_from_buf(const char* bwt_buf, 
+			  BowtieHit& bh,
+			  bool strip_slash,
+			  char* name_out = NULL,
+			  char* name_tags = NULL,
+			  char* seq = NULL,
+			  char* qual = NULL);
 
-	
-	void undo_hit() 
-	{ 
-		bgzf_seek(((samfile_t*)_hit_file)->x.bam, _curr_pos, SEEK_SET);
-		_eof = false;
-	}
-    */
-	string hitfile_rec(HitStream& hs, const char* hit_buf);
-
-	bool get_hit_from_buf(const char* bwt_buf, 
-			      BowtieHit& bh,
-			      bool strip_slash,
-			      char* name_out = NULL,
-			      char* name_tags = NULL,
-			      char* seq = NULL,
-			      char* qual = NULL);
-protected:
-	//int64_t _curr_pos;
-	//int64_t _beginning;
-	bam1_t _next_hit; 
-	bam_header_t* _sam_header;
+    void set_sam_header(bam_header_t* header) { _sam_header = header; }
+    
+ protected:
+    //int64_t _curr_pos;
+    //int64_t _beginning;
+    bam1_t _next_hit; 
+    bam_header_t* _sam_header;
     bool inspect_header(HitStream& hs);
 };
 
@@ -1413,6 +1245,8 @@ void add_hit_to_coverage(const BowtieHit& bh, vector<unsigned int>& DoC);
 
 void accept_all_hits(HitTable& hits);
 
+int gap_length(const vector<CigarOp>& cigar);
+
 //print BowtieHit as BAM record
 void print_bamhit(GBamWriter& wbam,
 		  const char* read_name,
