@@ -151,6 +151,7 @@ InsertAlignmentGrade(const BowtieHit& h1,
 
     //
     static const int penalty_for_long_inner_dist = bowtie2_max_penalty;
+    static const int penalty_for_discordant = bowtie2_max_penalty;
     alignment_score = h1.alignment_score() + h2.alignment_score();
     if (!fusion)
       {
@@ -175,6 +176,11 @@ InsertAlignmentGrade(const BowtieHit& h1,
 	if (!opposite_strands)
 	  alignment_score -= penalty_for_same_strand;
       }
+    else
+      {
+	if (!fusion_search)
+	  alignment_score -= penalty_for_discordant;
+      }
   }
   
   InsertAlignmentGrade& operator=(const InsertAlignmentGrade& rhs)
