@@ -680,16 +680,18 @@ BowtieHit merge_chain_color(RefSequenceTable& rt,
       if (found_closure)
   {
     bool end = false;
+    int mismatches = prev_hit->mismatches() + curr_hit->mismatches() + mismatch;
     BowtieHit merged_hit(reference_id,
 			 reference_id,
-             insert_id,
-             new_left,
-             new_cigar,
-             antisense,
-             antisense_closure,
-             prev_hit->edit_dist() + curr_hit->edit_dist() + mismatch,
-             prev_hit->splice_mms() + curr_hit->splice_mms(),
-             end);
+			 insert_id,
+			 new_left,
+			 new_cigar,
+			 antisense,
+			 antisense_closure,
+			 mismatches,
+			 mismatches + gap_length(new_cigar),
+			 prev_hit->splice_mms() + curr_hit->splice_mms(),
+			 end);
 
     if (curr_seg_index > 1)
       merged_hit.seq(seq.substr(first_seg_length + (curr_seg_index - 1) * segment_length, 2 * segment_length));
@@ -723,7 +725,7 @@ BowtieHit merge_chain_color(RefSequenceTable& rt,
   int num_splice_mms = 0;
   for (list<BowtieHit>::iterator s = hit_chain.begin(); s != hit_chain.end(); ++s)
     {
-      num_mismatches += s->edit_dist();
+      num_mismatches += s->mismatches();
       num_splice_mms += s->splice_mms();
 
       /*
@@ -777,14 +779,15 @@ BowtieHit merge_chain_color(RefSequenceTable& rt,
   bool end = false;
   BowtieHit new_hit(reference_id,
 		    reference_id,
-        insert_id,
-        left,
-        long_cigar,
-        antisense,
-        saw_antisense_splice,
-        num_mismatches,
-        num_splice_mms,
-        end);
+		    insert_id,
+		    left,
+		    long_cigar,
+		    antisense,
+		    saw_antisense_splice,
+		    num_mismatches,
+		    num_mismatches + gap_length(long_cigar),
+		    num_splice_mms,
+		    end);
 
   new_hit.seq(seq);
   new_hit.qual(qual);
@@ -1819,6 +1822,7 @@ BowtieHit merge_chain(RefSequenceTable& rt,
       if (found_closure)
 	{
 	  bool end = false;
+	  int mismatches = prev_hit->mismatches() + curr_hit->mismatches() + mismatch;
 	  BowtieHit merged_hit(prev_hit->ref_id(),
 			       curr_hit->ref_id2(),
 			       insert_id,
@@ -1826,7 +1830,8 @@ BowtieHit merge_chain(RefSequenceTable& rt,
 			       new_cigar,
 			       antisense,
 			       antisense_closure,
-			       prev_hit->edit_dist() + curr_hit->edit_dist() + mismatch,
+			       mismatches,
+			       mismatches + gap_length(new_cigar),
 			       prev_hit->splice_mms() + curr_hit->splice_mms(),
 			       end);
 
@@ -1887,7 +1892,7 @@ BowtieHit merge_chain(RefSequenceTable& rt,
   int num_splice_mms = 0;
   for (list<BowtieHit>::iterator s = hit_chain.begin(); s != hit_chain.end(); ++s)
     {
-      num_mismatches += s->edit_dist();
+      num_mismatches += s->mismatches();
       num_splice_mms += s->splice_mms();
 
       /*
@@ -1947,6 +1952,7 @@ BowtieHit merge_chain(RefSequenceTable& rt,
 		    antisense,
 		    saw_antisense_splice,
 		    num_mismatches,
+		    num_mismatches + gap_length(long_cigar),
 		    num_splice_mms,
 		    end);
 
@@ -2214,8 +2220,12 @@ bool dfs_seg_hits(RefSequenceTable& rt,
 		  size_t curr,
 		  vector<BowtieHit>& seg_hit_stack,
 		  vector<BowtieHit>& joined_hits,
+		  int& num_try,
 		  int fusion_dir = FUSION_NOTHING)
 {
+  if (num_try <= 0)
+    return false;
+  
   assert (!seg_hit_stack.empty());
   bool join_success = false;
 
@@ -2233,8 +2243,8 @@ bool dfs_seg_hits(RefSequenceTable& rt,
 	  BowtieHit* prevHit = &bh_prev;
 	  BowtieHit* currHit = &bh;
 
-	  // daehwan
-	  // if (bh.insert_id() == 11921733)
+	  // daehwan - for debugging purposes
+	  // if (bh.insert_id() == 792140)
 	  //  bDebug = true;
 
 	  /*
@@ -2547,6 +2557,7 @@ bool dfs_seg_hits(RefSequenceTable& rt,
 					  curr + 1,
 					  seg_hit_stack,
 					  joined_hits,
+					  num_try,
 					  dir == FUSION_NOTHING ? fusion_dir : dir);
 	      
 	      if (success)
@@ -2559,6 +2570,7 @@ bool dfs_seg_hits(RefSequenceTable& rt,
     }
   else
     {
+      --num_try;
       merge_segment_chain(rt,
 			  read_seq,
 			  read_quals,
@@ -2608,7 +2620,9 @@ bool join_segments_for_read(RefSequenceTable& rt,
 	seg_hit_stack.push_back(bh.reverse());
       else
 	seg_hit_stack.push_back(bh);
-      
+
+      const int max_try = 10000;
+      int num_try = max_try;
       bool success = dfs_seg_hits(rt,
 				  read_seq,
 				  read_quals,
@@ -2618,7 +2632,8 @@ bool join_segments_for_read(RefSequenceTable& rt,
 				  seg_hits_for_read, 
 				  1, 
 				  seg_hit_stack,
-				  joined_hits);
+				  joined_hits,
+				  num_try);
       if (success)
 	join_success = true;
       seg_hit_stack.pop_back();
@@ -2632,7 +2647,7 @@ struct JoinSegmentsWorker
   void operator()()
   {
     ReadTable it;
-    GBamWriter bam_writer(bam_output_fname.c_str(), sam_header_fname.c_str());
+    GBamWriter bam_writer(bam_output_fname.c_str(), sam_header_fname.c_str(), bam_output_fname + ".index");
     ReadStream readstream(reads_fname);
     if (readstream.file() == NULL)
       err_die("Error: cannot open %s for reading\n", reads_fname.c_str());
@@ -2799,6 +2814,11 @@ struct JoinSegmentsWorker
 		    joined_hits.erase(new_end, joined_hits.end());
 		    for (size_t i = 0; i < joined_hits.size(); i++)
 		      {
+			if (joined_hits[i].mismatches() > read_mismatches ||
+			    joined_hits[i].gap_length() > read_gap_length ||
+			    joined_hits[i].edit_dist() > read_edit_dist)
+			  continue;
+			    
 			const char* ref_name = rt->get_name(joined_hits[i].ref_id());
 			const char* ref_name2 = "";
 			if (joined_hits[i].fusion_opcode() != FUSION_NOTHING)
@@ -2880,8 +2900,8 @@ void driver(const string& bam_output_fname,
   fprintf (stderr, "Loading reference sequences...\n");
   get_seqs(ref_stream, rt, true);
     fprintf (stderr, "        reference sequences loaded.\n");
-
   fprintf(stderr, "Loading junctions...");
+  
   std::set<Junction> possible_juncs;
 
   for (size_t i = 0; i < possible_juncs_files.size(); ++i)
@@ -3073,7 +3093,7 @@ void driver(const string& bam_output_fname,
       
       worker.end_id = (i+1 < num_threads) ? read_ids[i] : std::numeric_limits<uint64_t>::max();
 
-      if (num_threads > 1)
+      if (num_threads > 1 && i + 1 < num_threads)
 	threads.push_back(new boost::thread(worker));
       else
 	worker();
