@@ -118,7 +118,7 @@ void read_best_alignments(const HitsForRead& hits_for_read,
 			  const FusionSet& fusions = empty_fusions,
 			  const Coverage& coverage = empty_coverage,
 			  bool final_report = false,
-			  boost::random::mt19937* rng = NULL)
+			  boost::mt19937* rng = NULL)
 {
   const vector<BowtieHit>& hits = hits_for_read.hits;
   if (hits.size() >= max_multihits * 5)
@@ -313,7 +313,7 @@ void pair_best_alignments(const HitsForRead& left_hits,
 			  const FusionSet& fusions = empty_fusions,
 			  const Coverage& coverage = empty_coverage,
 			  bool final_report = false,
-			  boost::random::mt19937* rng = NULL)
+			  boost::mt19937* rng = NULL)
 {
   const vector<BowtieHit>& left = left_hits.hits;
   const vector<BowtieHit>& right = right_hits.hits;
@@ -350,7 +350,7 @@ void pair_best_alignments(const HitsForRead& left_hits,
 
 	  // daehwan - for debugging purposes
 #if 0
-	  if (lh.insert_id() == 325708 && !g.is_fusion() && false)
+	  if (lh.insert_id() == 10790262)
 	    {
 	      fprintf(stderr, "lh %d:%d %s score: %d (from %d) NM: %d\n",
 		      lh.ref_id(), lh.left(), print_cigar(lh.cigar()).c_str(),
@@ -390,7 +390,7 @@ void pair_best_alignments(const HitsForRead& left_hits,
 
   // daehwan - for debugging purposes
 #if 0
-  if (best_hits.size() > 0 && best_hits[0].first.insert_id() == 37239044)
+  if (best_hits.size() > 0 && best_hits[0].first.insert_id() == 10790262)
     {
       for (size_t i = 0; i < best_hits.size(); ++i)
 	{
@@ -412,7 +412,7 @@ void pair_best_alignments(const HitsForRead& left_hits,
 
   // daehwan - for debugging purposes
 #if 0
-  if (best_hits.size() > 0 && best_hits[0].first.insert_id() == 37239044)
+  if (best_hits.size() > 0 && best_hits[0].first.insert_id() == 10790262)
     {
       for (size_t i = 0; i < best_hits.size(); ++i)
 	{
@@ -872,7 +872,7 @@ void print_sam_for_single(const RefSequenceTable& rt,
 			  FragmentType frag_type,
 			  const string& alt_name,
 			  GBamWriter& bam_writer,
-			  boost::random::mt19937& rng)
+			  boost::mt19937& rng)
 {
     //assert(!read.alt_name.empty());
     if (hits.hits.empty())
@@ -915,7 +915,7 @@ void print_sam_for_pair(const RefSequenceTable& rt,
                         GBamWriter& bam_writer,
                         const string& left_alt_name,
                         const string& right_alt_name,
-			boost::random::mt19937& rng,
+			boost::mt19937& rng,
 			uint64_t begin_id = 0,
                         uint64_t end_id = std::numeric_limits<uint64_t>::max())
 {
@@ -1653,12 +1653,19 @@ public:
 	  }
 	else
 	  {
-	    for (size_t j = 0; j < (size_t)num_threads; ++j)
+	    size_t j = 0;
+	    while (true)
 	      {
 		char suffix[128];
 		sprintf(suffix, "%lu.bam", j);
 		string temp_fname = fname + suffix;
+
+		ifstream index_file(temp_fname.c_str());
+		if (!index_file.is_open())
+		  break;
+
 		temp_fnames.push_back(temp_fname);
+		++j;
 	      }
 	  }
 
@@ -1684,10 +1691,13 @@ public:
 		  {
 		    offsets.push_back(last_offset);
 
+		    // daehwan - for debugging purposes
 #if 0
 		    fprintf(stderr, "bet %lu and %lu - %s %lu %ld\n",
 			    _begin_id, _end_id, temp_fnames[j].c_str(), last_offset, last_read_id);
 #endif
+
+		    break;
 		  }
 		
 		last_offset = offset;
@@ -1837,6 +1847,7 @@ struct ConsensusEventsWorker
 		// Process hit for right singleton, select best alignments
 		read_best_alignments(curr_right_hit_group, best_hits, *gtf_junctions);
 		// update_coverage(best_hits, *coverage);
+
 		update_junctions(best_hits, *junctions);
 		update_insertions_and_deletions(best_hits, *insertions, *deletions);
 		update_fusions(best_hits, *rt, *fusions);
@@ -1846,7 +1857,7 @@ struct ConsensusEventsWorker
 	    r_hs.next_read_hits(curr_right_hit_group);
 	    curr_right_obs_order = it.observation_order(curr_right_hit_group.insert_id);
 	  }
-        
+
 	// Since we have both left hits and right hits for this insert,
 	// Find the best pairing and print both
 	while (curr_left_obs_order == curr_right_obs_order &&
@@ -1888,7 +1899,7 @@ struct ConsensusEventsWorker
 	    update_junctions(best_right_hit_group, *junctions);
 	    update_insertions_and_deletions(best_right_hit_group, *insertions, *deletions);
 	    update_fusions(best_right_hit_group, *rt, *fusions);
-	            
+
 	    l_hs.next_read_hits(curr_left_hit_group);
 	    curr_left_obs_order = it.observation_order(curr_left_hit_group.insert_id);
             
@@ -1929,7 +1940,6 @@ struct ReportWorker
   {
     HitsForRead best_hits;
     best_hits.insert_id = curr_obs_order;
-    
     realign_reads(curr_hit_group, *rt, *junctions, *rev_junctions,
 		  *insertions, *deletions, *rev_deletions, *fusions);
     
@@ -2049,7 +2059,7 @@ struct ReportWorker
 	    right_hs.next_read_hits(curr_right_hit_group);
 	    curr_right_obs_order = it.observation_order(curr_right_hit_group.insert_id);
 	  }
-	
+
 	// Since we have both left hits and right hits for this insert,
 	// Find the best pairing and print both
 	while (curr_left_obs_order == curr_right_obs_order &&
@@ -2212,7 +2222,7 @@ struct ReportWorker
   int64_t right_reads_offset;
   int64_t right_map_offset;
 
-  boost::random::mt19937 rng;
+  boost::mt19937 rng;
 };
 
 void driver(const string& bam_output_fname,
@@ -2291,11 +2301,11 @@ void driver(const string& bam_output_fname,
 	num_threads = 1;
     }
 
-  JunctionSet vjunctions[num_threads];
-  InsertionSet vinsertions[num_threads];
-  DeletionSet vdeletions[num_threads];
-  FusionSet vfusions[num_threads];
-  Coverage vcoverages[num_threads];
+  vector<JunctionSet> vjunctions(num_threads);
+  vector<InsertionSet> vinsertions(num_threads);
+  vector<DeletionSet> vdeletions(num_threads);
+  vector<FusionSet> vfusions(num_threads);
+  vector<Coverage> vcoverages(num_threads);
   
   vector<boost::thread*> threads;
   for (int i = 0; i < num_threads; ++i)
